import { AppConfigService } from '@sec-spec/lib-ng-app-config';
import { HttpClient, HttpParams } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { map } from 'rxjs/operators';
import { Observable } from 'rxjs';
import { Schedule } from '@sec-spec/sis-model-schedule';

// As of version 5.10.0, tslint cannot recognize decorators in
// association with this rule.
/* tslint:disable:completed-docs */

@Injectable({
  providedIn: 'root'
})
export class ScheduleService {

  baseRoute: string;

  constructor(
    private appConfig: AppConfigService,
    private http: HttpClient
  ) {
    this.baseRoute = `${this.appConfig.baseUrl}/api/schedules`;
  }

  /** Create a schedule.
   * @param schedules New schedule(s) to be created.
   * @returns Ids of created schedules.
   */
  createSchedules(schedules: Schedule[] | Schedule): Observable<any[]> {
    const body = ScheduleService.serializeSchedules(schedules);
    return this.http
      .post(this.baseRoute, body)
      .pipe(
        map((data: any) => {
          return data;
        })
      );
  }

  /** Get a schedule.
   * @param id Primary key of the requested schedule.
   * @returns Schedule data.
   */
  getSchedule(id: number): Observable<Schedule> {
    return this.http.get(`${this.baseRoute}/${id}`).pipe(
      map((data: any) => {
        return Schedule.deserialize(data);
      })
    );
  }

  /** Get multiple schedules.
   * @param ids Primary keys of the requested schedules.
   * @returns Schedule data.
   */
  getSchedules(ids: number[]): Observable<Schedule[]> {
    let params = new HttpParams();
    ids.forEach((id: number) => {
      params = params.append('ids[]', id.toString());
    });
    return this.http.get(
      this.baseRoute,
      { params: params, }
    ).pipe(
      map((data: any) => {
        const asArray = Array.isArray(data) ? data : [data];
        const schedules: Schedule[] = [];
        for (const item of asArray) {
          schedules.push(Schedule.deserialize(item));
        }
        return schedules;
      })
    );
  }

  /** Serialize schedules for transmission to server.
   * @param schedules Scheduls to be serialized.
   * @returns Serialized schedules.
   */
  private static serializeSchedules(schedules: Schedule[] | Schedule): any[] {
    const records = Array.isArray(schedules) ? schedules : [schedules];
    const serialized = [];
    for (const schedule of records) {
      serialized.push(schedule.serialize());
    }
    return serialized;
  }

  /** Update schedules.
   * @param schedules Existing schedules to update.
   * @returns No data.
   */
  updateSchedules(schedules: Schedule[] | Schedule): Observable<any> {
    const body = ScheduleService.serializeSchedules(schedules);
    return this.http.put(this.baseRoute, body);
  }

}
